@@ -28,5 +28,5 @@ class AbstractResultsReader(ManagedPlugin):
         ManagedPlugin.__init__(self)
         self.format=results_format
 
-    def __call__(self, filename, res=None): #pragma:nocover
+    def __call__(self, filename, res=None, suffixes=[]): #pragma:nocover
         raise TypeError, "Method __call__ undefined in reader for format "+str(self.format)
