@@ -12,7 +12,7 @@ import pyutilib.component.core
 
 pyutilib.component.core.PluginGlobals.push_env( "coopr.opt" )
 
-from solver import *
+from async_solver import *
 import manager
 import local
 
