@@ -18,7 +18,7 @@ import re
 
 from coopr.opt.base import results
 from coopr.opt.base.formats import *
-from coopr.opt import SolverResults
+from coopr.opt import SolverResults, SolutionStatus, SolverStatus, TerminationCondition
 from pyutilib.component.core import alias
 import pyutilib.misc
 
@@ -35,10 +35,14 @@ class ResultsReader_sol(results.AbstractResultsReader):
         if not name is None:
             self.name = name
 
-    def __call__(self, filename, res=None, soln=None):
+    def __call__(self, filename, res=None, soln=None, suffixes=[]):
         """
         Parse a *.sol file
         """
+
+        # determine which suffixes we will bother loading into a solution.
+        extract_duals = "dual" in suffixes
+
         if res is None:
             res = SolverResults()
         #
@@ -71,12 +75,16 @@ class ResultsReader_sol(results.AbstractResultsReader):
         m = z[nopts + 1] # constraints
         x = []
         y = []
-        for i in xrange(m):
+        i = 0
+        while i < m:
             line = IN.readline()
-            y += [float(line)]
-        for i in xrange(n):
+            y.append(float(line))
+            i += 1
+        i = 0
+        while i < n:
             line = IN.readline()
-            x += [float(line)]
+            x.append(float(line))
+            i += 1
         objno = [0,0]
         line = IN.readline()
         if line:                    # WEH - when is this true?
@@ -92,18 +100,58 @@ class ResultsReader_sol(results.AbstractResultsReader):
         res.solver.message = pyutilib.misc.yaml_fix(res.solver.message)
         ##res.solver.instanceName = osrl.header.instanceName
         ##res.solver.systime = osrl.header.time
+        res.solver.status = SolverStatus.ok
+        soln_status = SolutionStatus.unknown
+        if (objno[1] >= 0) and (objno[1] <= 99):
+            objno_message = "OPTIMAL SOLUTION FOUND!"
+            res.solver.termination_condition = TerminationCondition.optimal
+            soln_status = SolutionStatus.optimal
+        elif (objno[1] >= 100) and (objno[1] <= 199):
+            objno_message = "Optimal solution indicated, but ERROR LIKELY!"
+            res.solver.termination_condition = TerminationCondition.optimal
+            res.solver.status = SolverStatus.warning
+            soln_status = SolutionStatus.optimal
+        elif (objno[1] >= 200) and (objno[1] <= 299):
+            objno_message = "INFEASIBLE SOLUTION: constraints cannot be satisfied!"
+            res.solver.termination_condition = TerminationCondition.infeasible
+        elif (objno[1] >= 300) and (objno[1] <= 399):
+            objno_message = "UNBOUNDED PROBLEM: the objective can be improved without limit!"
+            res.solver.termination_condition = TerminationCondition.unbounded
+        elif (objno[1] >= 400) and (objno[1] <= 499):
+            objno_message = "EXCEEDED MAXIMUM NUMBER OF ITERATIONS: the solver was stopped by a limit that you set!"
+            res.solver.termination_condition = TerminationCondition.maxIterations
+            soln_status = SolutionStatus.stoppedByLimit
+        elif (objno[1] >= 500) and (objno[1] <= 599):
+            objno_message = "FAILURE: the solver stopped by an error condition in the solver routines!"
+            res.solver.termination_condition = TerminationCondition.solverError
+            res.solver.status = SolverStatus.error
         res.solver.id = objno[1]
         ##res.problem.name = osrl.header.instanceName
-        if soln is None:
-            soln = res.solution.add()
-        ##soln.status = Solution.SolutionStatus(soln.status_type)
-        ##soln.status_description = soln.status_description
-        soln.message = msg.strip()
-        soln.message = res.solver.message.replace("\n","; ")
-        for i in range(0,len(x)):
-            soln.variable["v"+str(i)] = {"Value" : x[i], "Id" : len(soln.variable)}
-        for i in range(0,len(y)):
-            soln.constraint["c"+str(i)].dual = y[i]
+        if res.solver.termination_condition in [TerminationCondition.unknown,
+                        TerminationCondition.maxIterations,
+                        TerminationCondition.minFunctionValue,
+                        TerminationCondition.minStepLength,
+                        TerminationCondition.globallyOptimal,
+                        TerminationCondition.locallyOptimal,
+                        TerminationCondition.optimal,
+                        TerminationCondition.maxEvaluations,
+                        TerminationCondition.other]:
+
+            if soln is None:
+                soln = res.solution.add()
+            res.solution.status = soln_status
+            soln.status_description = objno_message
+            soln.message = msg.strip()
+            soln.message = res.solver.message.replace("\n","; ")
+            i = 0
+            soln_variable = soln.variable
+            for var_value in x:
+                soln_variable["v"+str(i)] = {"Value" : var_value, "Id" : i}
+                i += 1
+            if extract_duals is True:
+               for i in range(0,len(y)):
+                   soln.constraint["c"+str(i)].dual = y[i]
+
         #
         # This is a bit of a hack to accommodate PICO.  If
         # the PICO parser has parsed the # of constraints, then
